# BRYOCARB code by Clive Anderson, modified by Jennifer Kowalczyk and Clive Anderson.
# v2 - Apr 2017
#
# Implementation of the model presented in Fletcher et al. (2006) that estimates
# atmospheric CO2 from carbon isotope fractionation in liverwort photosynthesis.
#
# This version can be used for liverworts with or without pores as long as parameters are set appropriately, as described below. 
#
# INPUT PARAMETERS (see example files castlerock_withpore_inputs.csv and castlerock_nopore_inputs.csv):  The "_err" input should be standard deviation or standard error of the mean, 
# as appropriate.  If this value is unknown, the error can be estimated as some fraction of the mean (e.g., 5%).
#
# d13Cpl - measured carbon isotope composition of liverwort tissue (per mil)
# d13Cpl_err - standard error on above (per mil)
# diag_corr - correction to be applied to d13Cpl for diagenesis (per mil) (e.g., Fletcher et al., 2004)
# diag_err - uncertainty on above (per mil)
# d13Catm - atmospheric CO2 carbon isotope composition (per mil) (e.g., Tipple et al., 2010)
# d13Catm_err - uncertainty on above (per mil)
# d13Catm_corr - correction to be applied to d13Catm for canopy effects (per mil). Should be 0 for open environments.
# d13Catm_corr_err - uncertainty on above (per mil)
# p1 - slope of linear Ca-resistance relation (m^2 s/mol/ppm)
# p1_err - uncertainty on above (m^2 s/mol/ppm)
# p0 - y-intercept of linear Ca-resistance relation (m^2 s/mol)
# p0_err - uncertainty on above (m^2 s/mol)
# vm - maximum rate of carboxylation by Rubisco (umol/m^2/s)
# vm_err - uncertainty on above (umol/m^2/s)
# rd - dark respiration rate (umol/m^2/s)
# rd_err - uncertainty on above (umol/m^2/s)
# temp - estimate of paleotemperature (K)
# temp_err - uncertainty on above (K)
# o2 - paleo-atmospheric oxygen concentration (ppm) (e.g., Royer et al., 2014)
# o2_err - uncertainty on above (ppm)
# age - age of liverwort (Myrs)
# age_err - uncertainty on above (Myrs)
# latitude - paleolatitude of liverwort's growth site (degrees) (e.g., van Hinsbergen et al., 2015)
# lat_err - uncertainty on above (degrees)
# cloud - estimate of average cloud cover (unitless)
# cloud_err - uncertainty on above
# LAI - estimate of Leaf Area Index (unitless)
# LAI_err - uncertainty on above
# attenuation - attenuation coefficient, k (unitless)
# atten_err - uncertainty on above
#
# Values for p1, p0, vm, and rd for can be found Table S3 of Fletcher et al. (2008).  
# The first column of Table S3 (BRYOCARB_p) is appropriate for liverworts with pores and the second column (BRYOCARB_np) is for liverworts without pores.  
# Note in particular that p1 and p1_err should be set to zero for liverworts without pores.
#
# Optionally edit input and output file names, perentiles to be reported in summary file, and number of 
# Monte Carlo simulations to be run below ("EDIT AS INDICATED BELOW").  The code iterates until reaching the
# desired number of successful simulations (nsim).
# Note that it is assumed that input file is in the working directory.
#
# Run code with "Ctrl+Alt+r" or Code > Run Region > Run All.
#
# References:
#
# Fletcher, B. J., Beerling, D. J., and Chaloner, W., 2004, Stable carbon isotopes and the metabolism of the terrestrial Devonian organism Spongiophyton: Geobiology, v. 2, p. 107-119.
# Fletcher, B. J., Beerling, D. J., Brentnall, S. J., and Royer, D. L., 2005, Fossil bryophytes as recorders of ancient CO2 levels: experimental evidence and a Cretaceous case study: Global Biogeochemical Cycles, v. 19, GB3012.
# Fletcher, B. J., Brentnall, S. J., Quick, W. P., and Beerling, D. J., 2006, BRYOCARB: A process-based model of thallose liverwort carbon isotope fractionation in response to CO2, O2, light and temperature: Geochimica et cosmochimica acta, v. 70, p. 5676-5691.
# Fletcher, B. J., Brentnall, S. J., Anderson, C. W., Berner, R. A., and Beerling, D. J., 2008, Atmospheric carbon dioxide linked with Mesozoic and early Cenozoic climate change: Nature Geoscience, v. 1, p. 43-48. 
# Royer, D. L., Donnadieu, Y., Park, J., Kowalczyk, J., and Godd?ris, Y., 2014, Error analysis of CO2 and O2 estimates from the long-term geochemical model GEOCARBSULF: American Journal of Science, v. 314, p. 1259-1283.
# Tipple, B. J., Meyers, S. R., and Pagani, M., 2010, Carbon isotope ratio of Cenozoic CO2: A comparative evaluation of available geochemical proxies: Paleoceanography, v. 25, p. PA3202.
# van Hinsbergen, D. J. J., de Groot, L. V., van Schaik, S. J., Sluijs, A., Bijl, P. K., Spakman, W., Langereis, C. G., and Brinkhuis, H., 2015, A paleolatitude calculator for paleoclimate studies: PLOS ONE, v. 10, e0126946.
#
###############################################################
############### EDIT AS INDICATED BELOW #######################

# Optionally edit input and output file names
infilename="bryocarb_example_withpore_input.csv"
outfilename="bryocarb_withpore_output_summary.csv"
outsimfile="bryocarb_withpore_output_CO2_sims.csv"

# Optionally edit percentiles to be reported (e.g., 2.5 and 97.5 for 95% confidence interval)
# and number of Monte Carlo simulations to run for each input
low_percentile <- 2.5; low_percentile_label <- paste0(low_percentile,"_percentile")  
high_percentile <- 97.5; high_percentile_label <- paste0(high_percentile,"_percentile")
nsim <- 10000;

# Set how irradiance error is calculated.  If set to yes, error is estimated as 12.5% of the mean value.  If not set to yes, it is estimated from Monte Carlo propogation of the uncertainties on input parameters used to calculate irradiance. 
irr_err_fixed="yes"

############ END OF SECTION TO EDIT ############################
################################################################

# bryosimp is main function.  Sets up distributions for each input paramter, runs MC simulations, reports results.
"bryosimp" <-
function(idno,means,errs,corr1,corr2,nsim)
{
#allows beta (p0) to be correlated with vm 
# modified to signal infeasible input combinations 
# and correlated D13 & q   27/06/06
runs2=numeric(nsim) #array for output Ca values
Aout=numeric(nsim) #array for modeled photosynthetic rates
nsim2=nsim*5 #nsim2 > nsim because some runs will fail (i.e. unfeasible Ca or photosynthetic rates)
d13Catm=rnorm(nsim2,(means[3]+means[4]),sqrt(errs[3]^2 + errs[4]^2)) # atm. CO2 d13C
sampleD13C=calc_sample_D13C(means[1:4],errs[1:4]) # calculate sample D13C
irrad=calc_irrad(c(means[11],means[12],means[13],means[14],means[15]),c(errs[11:15]),irr_err_fixed) # calculate ln(irradiance)
corradj=corr2*(sqrt(exp(irrad[2]^2)-1)/irrad[2]) # adjust correlation matrix for log-normality
corradj[1,1] <- 1 #return diagonal elements to 1
corradj[2,2] <- 1 #return diagonal elements to 1
D13q=rmvnorm(nsim2,2,c(sampleD13C[1],irrad[1]),c(sampleD13C[2],irrad[2]),corradj) # set up bivariate normal dist. for D13C and ln(irradiance)
D13=D13q[,1] # sample carbon isotope fractionation
q=exp(D13q[,2]) # irradiance (PAR)
alp=rnorm(nsim2,means[5],errs[5]) #p1
betvm=rmvnorm(nsim2,2,means[6:7],errs[6:7],corr1) # set up bivariate normal dist. for p0 and Vcmax
bet=betvm[,1] #p0
vm=betvm[,2] #Vcmax
rd=rnorm(nsim2,means[8],errs[8]) # dark respiration rate
temp=rnorm(nsim2,means[9],errs[9]) # temperature
oi=rnorm(nsim2,means[10],errs[10]) # paleo atm o2 concentration
counts=numeric(4) #for error reporting
goodruns=0
i=1
while(goodruns < nsim) { #continue MC simulations until nsim good runs
 if(i>nsim2){cat("End of iterations reached before producing the desired amount of good runs.")}
 ca = invfun2(D13[i],alp[i],bet[i],vm[i],rd[i],temp[i],oi[i],q[i],d13Catm[i]) #Ca value
 # is run successful?
 if(is.na(ca[1])){counts[1] = counts[1] + 1} #run is unsuccessful if sample D13C < min feasible D13C
 if(!is.na(ca[1])&&ca[1]>1000000){counts[2] = counts[2] + 1; ca[1]=NA} # run is unsuccessful if Ca > 10^6
 if(!is.na(ca[1])&&ca[1]<0){counts[3] = counts[3] + 1; ca[1]=NA} # run is unsuccessful if Ca < 0
 if(!is.na(ca[1])){ # if Ca != NA, get photosynthetic rate for this run
 blah=calc_pred_D13C(ca[1],alp[i],bet[i],vm[i],rd[i],temp[i],oi[i],q[i],d13Catm[i]) 
 aval=blah[2]
  }
 if(!is.na(ca[1])&&aval<0){counts[4]=counts[4]+1;ca[1]=NA} # run is unsuccessful if photo. rate < 0
 if(!is.na(ca[1])){ #if run is successful, save Ca and photo. rate
   runs2[(goodruns+1)]=ca[1]
   Aout[(goodruns+1)]=aval
   goodruns = goodruns +1}
 if(i==1){cat("\n\nINPUT ROW ",j,"\n")}
 if((i - 1000 * floor(i/1000))==0){cat("iteration ",i,"\n")}
 i = i+1
}
cat("Good runs: ",goodruns," Total runs: ",i-1," (",(goodruns/(i-1))*100,"%)\n")
cat("Sample D13C too low: ",counts[1],"\n")
cat("Ca > 1000000: ",counts[2],"\n")
cat("Ca < 0: ",counts[3],"\n")
cat("Photosynthetic rate < 0: ",counts[4],"\n\n")
sdCa=sqrt(var(runs2))
esestmean=sdCa/sqrt(nsim)
mnCa=mean(runs2)
ranCa=range(runs2)
medCa=median(runs2)
lowCa=quantile(runs2, probs=low_percentile/100, na.rm=TRUE)
highCa=quantile(runs2, probs=high_percentile/100, na.rm=TRUE)
cat("Median = ",medCa," ppm\n")
cat(low_percentile," percentile = ",lowCa," ppm\n")
cat(high_percentile," percentile = ",highCa," ppm\n")
cat("Mean photosynthetic rate = ",mean(Aout),"umol/m^2/s\n")
cat("Std. dev. photosynthetic rate = ",sd(Aout),"umol/m^2/s\n")
cat("Sample D13C = ",mean(D13)," +/- ",sd(D13)," per mil\n")
cat("Irradiance = ",mean(q)," +/- ",sd(q)," umol/m^2/s PAR\n\n\n") 
runden=density(runs2)
par(mfrow=c(2,2))
plot(runden,xlim=c(0,highCa+100)) # plot probability density function
fwdfun(c(means[5:10],mean(q),mean(d13Catm))) # plot transfer function (D13C vs Ca)
par(mfrow=c(1,1))
out <- c(mnCa,esestmean,sdCa,medCa,lowCa,highCa,mean(Aout),sd(Aout),i-1,runs2)
out
}


# calculate sample D13C
"calc_sample_D13C" <-
function(means,errs)
{
d13Cpl=means[1]
d13Cpl_err=errs[1]
diag_corr=means[2]
diag_corr_err=errs[2]
d13Catm=means[3]
d13Catm_err=errs[3]
d13Catm_corr=means[4]
d13Catm_corr_err=errs[4]

# sample D13C
D13Cin=((d13Catm+d13Catm_corr)-(d13Cpl+diag_corr))/(1+(d13Cpl+diag_corr)/1000)

# propagate errors

# partial derivatives
dCdp = -1*((d13Catm + d13Catm_corr - diag_corr - d13Cpl)/(1000*(1+(d13Cpl+diag_corr)/1000)^2) + 1/(1+(d13Cpl+diag_corr)/1000))
dCddiag=-1*((d13Catm + d13Catm_corr - diag_corr - d13Cpl)/(1000*(1+(d13Cpl+diag_corr)/1000)^2) + 1/(1+(d13Cpl+diag_corr)/1000))
dCdatm = 1/(1+(d13Cpl+diag_corr)/1000)
dCdatmcorr = 1/(1+(d13Cpl+diag_corr)/1000)

D13Cvar=d13Cpl_err^2*dCdp^2 + diag_corr_err^2*dCddiag^2 + d13Catm_err^2*dCdatm^2 + d13Catm_corr_err^2*dCdatmcorr^2
D13Cin_err=sqrt(D13Cvar)

out <- c(D13Cin, D13Cin_err)
out
}


# Hard-coded correlations between p0 and vm (corr 1) and between D13C and irradiance (corr2)
"corr1" <-
  structure(c(1, -0.78, -0.78, 1), .Dim = as.integer(c(2, 2)))
"corr2" <-
  structure(c(1, -0.5, -0.5, 1), .Dim = as.integer(c(2, 2)))


#generates truncated multivariate normal distribution (truncated so that 1st > 0.5) 
"rmvnorm" <-
function(nsim,d,mean,errs,corr)
{
  v=matrix(0,d,d)
  diag(v)<-errs
  vcv=v%*%corr%*%v #Note: "%*%" is matrix multiplication
  r=chol(vcv) #choleski
  out=matrix(0,nsim,d)
  for (i in 1:nsim) {
  repeat {
  z=matrix(rnorm(d),d,1)
  out[i,]=mean+t(r)%*%z 
  if (out[i,1] > 0.5) {break }
  }
     }
  out
}


# Estimate Ca by constructing Ca vs. model-predicted D13C curve, then
# finding value of Ca corresponding to sample D13C
"invfun2" <-
function(D13,alph,bet,vm,rd,temp,oi,q,d13Catm)
{
m=c(D13,alph,bet,vm,rd,temp,oi,q,d13Catm)
flag=1
d13ends=min.binse2(10,1000,m,0.000001) #find min. feasible D13C for Ca between 10-1000 ppm
if (D13 < d13ends[2])  { # sample D13C lower than min. feasible value
Ca=NA}
else  {
cavals=seq(d13ends[1],6000,by=25) #set up array of Ca values from min to 6000 pppm
D13vals=apply(array(cavals),1,calc_pred_D13C,alph,bet,vm,rd,temp,oi,q,d13Catm) # find D13C value for each Ca
if (any(D13vals[2,]<0)) { flag=-1 } #set flag if photosynthetic rate < 0
is.function(f <- splinefun(D13vals[1,],cavals)) #spline curve of D13C vs Ca
Ca=f(D13)  } # find value of Ca corresponding to sample D13C
Ca=c(Ca,d13ends,flag)
Ca
}


#Find min D13C for given input values, and corresponding Ca
"min.binse2" <-
function(a,b,m,tol)
{
#  carries out a binary search for min of D13C
#  a<b are values between which min believed to lie
#  corrects a bug in min.binse   27/06/06
alph=m[2]
bet=m[3]
vm=m[4]
rd=m[5]
temp=m[6]
oi=m[7]
q=m[8]
d13Catm=m[9]
    diffint = abs(b - a)
    while(diffint >= tol) {
        xmid = (a + b)/2
        f1 = calc_pred_D13C(xmid,alph,bet,vm,rd,temp,oi,q,d13Catm)
        f2 = calc_pred_D13C(xmid+tol,alph,bet,vm,rd,temp,oi,q,d13Catm)
        if(f1[1] > f2[1]) { #fn[1] is calculated D13C, fn[2] is photosynthetic rate
            a = xmid
        }
        else {
            b = xmid
        }
        diffint = abs(b - a)
        }
    xmid=(a+b)/2
    d13=calc_pred_D13C(xmid,alph,bet,vm,rd,temp,oi,q,d13Catm)[1] #[1] is calculated D13C
    out=c(xmid,d13)
    # xmid is value of ca at which D13C is min, and d13 is min value 
    out
}


# calculate model-predicted D13C for given Ca and input parameter values
"calc_pred_D13C" <-
function(ca,alph,bet,vm,rd,temp,oi,q,d13Catm)
{R=8.314
d13ca=d13Catm #note: in original code (e.g. Fletcher et al., 2008), d13ca hardcoded to -8 per mil
RT=R*temp
kc=10*exp(35.8-80500/RT)
k0=(10^4)*exp(9.6-14510/RT)
tau=exp(-3.949+28990/RT)
Gamstar=44.7+1.88*(temp-298.13)+0.036*(temp-298.13)^2
jm=1.64*vm+29.1
vmaxval=vmax(vm,temp)
jmaxval=jmax(jm,temp)
J=0.18*q*jmaxval/sqrt(jmaxval^2+(0.18*250)^2) # Fletcher et al. (2006) Equation 7
ci1val=ci1(ca,alph,bet,vmaxval,rd,kc,k0,oi,tau)
ci2val=ci2(alph,bet,J,oi,tau,rd,ca)
gamma1=1-oi/(2*tau*ci1val)
gamma2=1-oi/(2*tau*ci2val)
acval=ac(ci1val,vmaxval,gamma1,kc,k0,oi,rd)
aival=ai(gamma2,J,ci2val,oi,tau,rd)
if (acval < aival) { #set "aval" (photosynthetic rate) to min. of light and Rubisco limited values
aval = acval
ci= ci1val }
else {
aval = aival
ci=ci2val }

# carbon isotope fractionations (per mil) due to diffusion (a), Rubisco (b), dark respiration (e), and photorespiration (f)
# see Fletcher et al. (2006) Appendix A
a=4.4
b=30
f=2
e=7

d13cp=d13ca-a-(b-a)*(ci/ca) + (f*Gamstar+e*rd*(ci-Gamstar)/(aval+rd))/ca # Fletcher et al. (2006), Eq. 1
D13C=(d13ca - d13cp)/(1 + d13cp/1000)
out=c(D13C,aval)
out
}


# According to Fletcher et al. (2006), the temperature dependencies of Vmax and Jmax below 25 C are from
# Long (1991) and above 25 C are from Beerling & Woodward (2001).

# calculate Vmax for given temperature
"vmax" <-
function(vm,temp)
{
T25=298.15
R=8.314
dT=temp-T25
if (temp<=T25) {
vmax=vm*(sqrt(temp/T25))*exp(68000*dT/(R*temp*T25))
 }
else  {
vmax=vm*(1+0.0505*dT-0.248*(10^-3)*(dT^2)-8.09*(10^-5)*(dT^3))
 }
vmax
}


# calculate Jmax for given temperature
"jmax" <-
function(jm,temp)
{
T25=298.15
R=8.314
dT=temp-T25
if (temp<=T25) {
jmax=jm*(sqrt(temp/T25))*exp(28000*dT/(R*temp*T25))
 }
else  {
jmax=jm*(1+0.0409*dT-1.540*(10^-3)*(dT^2)-9.42*(10^-5)*(dT^3))
 }
jmax
}


# Ci for Rubisco-limited case
"ci1" <- 
function(ca,alph,bet,vm,rd,kc,k0,oi,tau)
{
    r=alph*ca+bet  #Fletcher et al. (2006) Equation 16; alph=p1, bet=p0
    w=r*vm-(r*rd+ca)+kc*(1+oi/k0) #Fletcher et al. (2006) Equation 11 
    x=-(r*oi*vm/(2*tau)+(r*rd+ca)*kc*(1+oi/k0)) #Fletcher et al. (2006) Equation 12
    ci1=(-w+sqrt(w^2-4*x))/2 #Fletcher et al. (2006) Equation 10
    ci1
}


# Ci for light-limited case
"ci2" <- 
function(alph, bet, J, oi, tau, rd, ca)
{
        r = alph*ca + bet #Fletcher et al. (2006) Equation 16; alph=p1, bet=p0
        y = r*J - 4*(r*rd + ca) + (4*oi)/tau  #Fletcher et al. (2006) Equation 14
        z =  - ((r*oi*J)/(2*tau) + (r*rd + ca)*(4*oi)/tau) #Fletcher et al. (2006) Equation 15
        ci2 = (-y + sqrt(y^2 - 16*z))/8 #Fletcher et al. (2006) Equation 13
        ci2
}


#Rubisco-limited photosynthetic rate
"ac" <- 
  function(ci,vm,gamma,kc,k0,oi,rd)
{
    ac=gamma*vm*ci/(ci+kc*(1+oi/k0))-rd #Fletcher et al. (2006) Equation 5
    ac
}


#light-limited photosynthetic rate
"ai" <- 
  function(gamma,J,ci,oi,tau,rd)
{
    ai=(gamma*J*ci/(4*(ci+oi/tau)))-rd #Fletcher et al. (2006) Equation 6
    ai
}


# plot D13C vs Ca
"fwdfun" <-
  function(m)
{
    alph=m[1]
    bet=m[2]
    vm=m[3]
    rd=m[4]
    temp=m[5]
    oi=m[6]
    q=m[7]
    d13Catm=m[8]
    cavals=seq(150,5000,by=25)
    D13vals=apply(array(cavals),1,calc_pred_D13C,alph,bet,vm,rd,temp,oi,q,d13Catm)
    plot(cavals,D13vals[1,],type="l",ylab=expression(Delta^13*C),xlab=expression("C"[a]))
    title("Fwd function")
    out=list(cavals=cavals,D13vals=D13vals)
    out
}

# calculate forest-floor irradiance, propagating uncertainties via Monte Carlo simulations
"calc_irrad" <-
  function(means,errs,irr_err_fixed)
{
  n=1000
  irrad_sim=numeric(n)  #output
  age=rnorm(n,means[1],errs[1])
  lat=rnorm(n,means[2],errs[2])
  cloud=rnorm(n,means[3],errs[3])
  lai=rnorm(n,means[4],errs[4])
  atten=rnorm(n,means[5],errs[5])
  
  for(i in 1:n){
  # calculate mid-day (max) solar irradiance
  solar = 600*(1 - (0.055*(age[i]/542.)))
  c <- 0:24
  mxm = 180
  tsn = 12
  sig = -23.5*cos((360*((mxm + 10)/365))*(pi/180))
  costc = sin(lat[i]*(pi/180))*sin(sig*(pi/180)) + cos(lat[i]*(pi/180))*cos(sig*(pi/180))*cos(15*(pi/180)*(c - tsn))
  cosh = -1*tan(lat[i]*(pi/180))*tan(sig*(pi/180))
  if(cosh > 1){cosh=0.999999}
  if(cosh < -1){cosh = -0.999999}
  
  betac = asin(costc)
  cotitc = 1/tan(betac)
  
  #Daylength calculation:
  dl = 2*(acos(cosh)/(pi/180))/15;
  if(lat[i] < 0){dl = 24 - dl}
  
  # Direct beam solar radiation:
  pdrc = solar*exp(-0.185/costc)*costc
  
  # Diffuse solar radiation:
  pdifc = (0.4*(solar - pdrc)*costc)*4.6
  for(ii in 1:length(pdifc)){
    if(pdifc[ii]<0){pdifc[ii]=0}
  }
  
  ang = 60
  soilalbedo = 0.4
  
  for (ii in 1:length(pdrc)){
    if(pdrc[ii]<0){pdrc[ii]=0}
  }
  
  pdrc=pdrc*4.6
  
  ppdifc = (cos((ang/2)*(pi/180))^2 + sin((ang/2)*(pi/180))^2)*pdifc + (cos((ang/2)*(pi/180))^2 + sin((ang/2)*(pi/180))^2)*((pdrc + pdifc)*soilalbedo)
  
  int = (pi/2)*cos(ang*(pi/180))-(1/tan(betac))*sin(ang*(pi/180))
  p45c=abs(int)/(pi/2)                                                  
  pp45c = p45c*pdrc + ppdifc
  for (ii in 1:length(pp45c)){
    if(betac[ii]<0){pp45c[ii]=0}
  }
  
  par=max(pp45c)
  
  # reduce for cloud and canopy cover
  par = par*cloud[i]
  irrad_sim[i]=par*exp(-1*atten[i]*lai[i]) #Beer's law
  }

  if (mean(irrad_sim) < 40){
    cat("Irradiance = ",irrad," too low for model!")
    stop()
  }
  
  if (irr_err_fixed == "yes") {stddev=0.125*mean(irrad_sim)} else {stddev=sd(irrad_sim)}
#  cat("Irr calc: ",mean(irrad_sim)," ",stddev)

  lnirr_mn=log((mean(irrad_sim)^2)/sqrt(stddev^2+mean(irrad_sim)^2))
  lnirr_err=sqrt(log((mean(irrad_sim)^2 + stddev^2)/(mean(irrad_sim)^2)))
  
  out<-c(lnirr_mn,lnirr_err)
  out
}

# Read in input file
input <- read.csv(infilename,header=TRUE) #read input file

d13Cpl_mean <- input[,"d13Cpl"]
d13Cpl_err <- input[,"d13Cpl_err"]
diag_corr <- input[,"diag_corr"]
diag_err <- input[,"diag_err"]
d13Catm_mean <- input[,"d13Catm"]
d13Catm_err <- input[,"d13Catm_err"]
d13Catm_corr <- input[,"d13Catm_corr"]
d13Catm_corr_err <- input[,"d13Catm_corr_err"]
p1_input <- input[,"p1"]
p1_err_input <- input[,"p1_err"]
p0_input <- input[,"p0"]
p0_err_input <- input[,"p0_err"]
vm_input <- input[,"vm"]
vm_err_input <- input[,"vm_err"]
rd_input <- input[,"rd"]
rd_err_input <- input[,"rd_err"]
temp_input <- input[,"temp"]
temp_err_input <- input[,"temp_err"]
o2_input <- input[,"o2"]
o2_err_input <- input[,"o2_err"]
age <- input[,"age"]
age_err <-input[,"age_err"]
lat <- input[,"latitude"]
lat_err <- input[,"lat_err"]
cloud <- input[,"cloud"]
cloud_err <- input[,"cloud_err"]
lai <- input[,"LAI"]
lai_err <- input[,"LAI_err"]
atten <- input[,"attenuation"]
atten_err <- input[,"atten_err"]

sampleN <- length(!is.na(atten))  #determine the number of samples (=rows)

# For output
CO2.summary <- matrix(nrow=sampleN, ncol=9, dimnames=list(input$sample,c("mean","stderr","stddev","median",low_percentile_label,high_percentile_label,"mean Aout","stddev Aout","runs")))
CO2.runs <- matrix(ncol=sampleN, nrow=nsim)

# Run code
for (j in 1:sampleN){  
  means <- c(d13Cpl_mean[j],diag_corr[j],d13Catm_mean[j],d13Catm_corr[j],p1_input[j],p0_input[j],vm_input[j],rd_input[j],temp_input[j],o2_input[j],age[j],lat[j],cloud[j],lai[j],atten[j])
  errs <- c(d13Cpl_err[j],diag_err[j],d13Catm_err[j],d13Catm_corr_err[j],p1_err_input[j],p0_err_input[j],vm_err_input[j],rd_err_input[j],temp_err_input[j],o2_err_input[j],age_err[j],lat_err[j],cloud_err[j],lai_err[j],atten_err[j])
  out.tmp = bryosimp(1,means,errs,corr1,corr2,nsim)
  CO2.summary[j,]=out.tmp[1:9]
  CO2.runs[,j]=out.tmp[10:length(out.tmp)]
}  

# Write output files
write.csv(CO2.summary, outfilename)
write.csv(CO2.runs,outsimfile)